------------------------------------------
-- Initial variables
------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("CalendarFu")
local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local endColor = "|r"
local newLine = "\n"

CalendarFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceConsole-2.0", "AceEvent-2.0", "AceDB-2.0")
CalendarFu.hasIcon = true
CalendarFu.independentProfile = true
CalendarFu.defaultPosition = "LEFT"
CalendarFu.defaultMinimapPosition = 180
CalendarFu.cannotDetachTooltip = true
CalendarFu.tooltipType = "Tablet-2.0"
CalendarFu.clickableTooltip = true
CalendarFu:RegisterDB("CalendarFuDB", "CalendarFuDBPC")

CalendarFu:RegisterDefaults("char", {new = 0, total = 0})
CalendarFu:RegisterDefaults("profile", {
	fordays = 3,
	evBlizzard = true,
	evRaid = true,
	evDungeon = true,
	evPvP = true,
	evMeeting = true,
	evOther = true,
	calPlayer = true,
	calGuild = true,
	calArena = true,
	calHoliday = true,
	notifyInvited = true,
	notifyAccepted = true,
	notifyDeclined = true,
	notifyConfirmed = true,
	notifyOut = true,
	notifyStandby = true,
	notifySigned = true,
	notifyNotsigned = true,
	notifyTentative = true,
	delimiterColor = {
		r = 1,
		g = 0.67450980392156,
		b = 0.35686274509803,
		a = 1,
	},
	dateColor = {
		r = 0.35686274509803,
		g = 0.54901960784313,
		b = 0.79607843137254,
		a = 1,
	},
	timeColor = {
		r = 1,
		g = 0.69411764705882,
		b = 0.36078431372549,
		a = 1,
	},
	titleColor = {
		r = 1,
		g = 1,
		b = 0,
		a = 1,
	},
})

------------------------------------------
-- Local variables
------------------------------------------

local ColorEventInv = {
	"|cff44bbbb", -- Teal: invited
	"|cff00ff00", -- Green: accepted
	"|cffff0000", -- Red: declined
	"|cff00cc00", -- Dark green: confirmed
	"|cffcc0000", -- Dark red: out
	"|cff0000ff", -- Blue: standby
	"|cff00ff00", -- Green: signed
	"|cff44bbbb", -- Teal: notsigned
	"|cfff26522", -- Orange: tentative
}

local eventInfo = {}

local iconFiles = {
	icon = "Interface\\AddOns\\FuBar_CalendarFu\\icon",
	iconnew = "Interface\\AddOns\\FuBar_CalendarFu\\iconnew",
}

------------------------------------------
-- Options
------------------------------------------

function CalendarFu:OnMenuRequest(level, value, inTooltip)
	if (level == 1) then
	    dewdrop:AddLine(
     		'text', "Open Calendar",
   			'tooltipTitle', "Open Calendar",
            'tooltipText', "Open Calendar window",
            'func', function()
				Calendar_Toggle()
				self:Update()
				end
		)
		dewdrop:AddLine()
		dewdrop:AddLine(
		    'text', "Days forward",
		    'tooltipTitle', "Days forward",
		    'tooltipText', "How much forward to scan for events",
		    'hasArrow', true,
		    'hasSlider', true,
		    'sliderMax', 30,
		    'sliderValue', self.db.profile.fordays,
		    'sliderStep', 1,
		    'sliderFunc', function(value)
				self.db.profile.fordays = value
				self:Update()
				end
		)
		dewdrop:AddLine()
		dewdrop:AddLine(
		    'text', "Event types",
		    'tooltipTitle', "Event Types",
		    'tooltipText', "Event Types",
		    'hasArrow', true,
		    'value', "eventTypes"
		)
		dewdrop:AddLine(
		    'text', "Calendar types",
		    'tooltipTitle', "Calendar Types",
		    'tooltipText', "Calendar Types",
		    'hasArrow', true,
		    'value', "calendarTypes"
		)
		dewdrop:AddLine(
		    'text', "Status types",
		    'tooltipTitle', "Status Types",
		    'tooltipText', "Types of invite status to show",
		    'hasArrow', true,
		    'value', "statusTypes"
		)
		dewdrop:AddLine(
			'text', "Tooltip coloring",
			'tooltipTitle', "Tooltip coloring",
			'tooltipText', "Tooltip coloring options.",
			'hasArrow', true,
			'value', "colorOptions"
		)
		dewdrop:AddLine()
    elseif (level == 2) then
	    if (value == "eventTypes") then
			dewdrop:AddLine(
	  			'text', "Blizzard",
	  			'tooltipTitle', "Blizzard",
				'tooltipText', "Blizzard",
				'checked', self.db.profile.evBlizzard,
				'func', function()
					self:ToggleEvBlizzard()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Raid",
	  			'tooltipTitle', "Raid",
				'tooltipText', "Raid",
				'checked', self.db.profile.evRaid,
				'func', function()
					self:ToggleEvRaid()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Dungeon",
	  			'tooltipTitle', "Dungeon",
				'tooltipText', "Dungeon",
				'checked', self.db.profile.evDungeon,
				'func', function()
					self:ToggleEvDungeon()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "PvP",
	  			'tooltipTitle', "PvP",
				'tooltipText', "Player vs. Player",
				'checked', self.db.profile.evPvP,
				'func', function()
					self:ToggleEvPvP()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Meeting",
	  			'tooltipTitle', "Meeting",
				'tooltipText', "Meeting",
				'checked', self.db.profile.evMeeting,
				'func', function()
					self:ToggleEvMeeting()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Other",
	  			'tooltipTitle', "Other",
				'tooltipText', "Other",
				'checked', self.db.profile.evOther,
				'func', function()
					self:ToggleEvOther()
					self:Update()
					end
			)
		elseif (value == "calendarTypes") then
			dewdrop:AddLine(
	  			'text', "Player",
	  			'tooltipTitle', "Player",
				'tooltipText', "Player",
				'checked', self.db.profile.calPlayer,
				'func', function()
					self:ToggleCalPlayer()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Guild",
	  			'tooltipTitle', "Guild",
				'tooltipText', "Guild",
				'checked', self.db.profile.calGuild,
				'func', function()
					self:ToggleCalGuild()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Arena",
	  			'tooltipTitle', "Arena",
				'tooltipText', "Arena",
				'checked', self.db.profile.calArena,
				'func', function()
					self:ToggleCalArena()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Holiday",
	  			'tooltipTitle', "Holiday",
				'tooltipText', "Holiday",
				'checked', self.db.profile.calHoliday,
				'func', function()
					self:ToggleCalHoliday()
					self:Update()
					end
			)
        elseif (value == "statusTypes") then
			dewdrop:AddLine(
	  			'text', "Invited",
	  			'tooltipTitle', "Invited",
				'tooltipText', "Invited",
				'checked', self.db.profile.notifyInvited,
				'func', function()
					self:ToggleNotifyInvited()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Accepted",
	  			'tooltipTitle', "Accepted",
				'tooltipText', "Accepted",
				'checked', self.db.profile.notifyAccepted,
				'func', function()
					self:ToggleNotifyAccepted()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Tentative",
	  			'tooltipTitle', "Tentative",
				'tooltipText', "Tentative",
				'checked', self.db.profile.notifyTentative,
				'func', function()
					self:ToggleNotifyTentative()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Declined",
	  			'tooltipTitle', "Declined",
				'tooltipText', "Declined",
				'checked', self.db.profile.notifyDeclined,
				'func', function()
					self:ToggleNotifyDeclined()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Confirmed",
	  			'tooltipTitle', "Confirmed",
				'tooltipText', "Confirmed",
				'checked', self.db.profile.notifyConfirmed,
				'func', function()
					self:ToggleNotifyConfirmed()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Out",
	  			'tooltipTitle', "Out",
				'tooltipText', "Out",
				'checked', self.db.profile.notifyOut,
				'func', function()
					self:ToggleNotifyOut()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Standby",
	  			'tooltipTitle', "Standby",
				'tooltipText', "Standby",
				'checked', self.db.profile.notifyStandby,
				'func', function()
					self:ToggleNotifyStandby()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Signed",
	  			'tooltipTitle', "Signed",
				'tooltipText', "Signed",
				'checked', self.db.profile.notifySigned,
				'func', function()
					self:ToggleNotifySigned()
					self:Update()
					end
			)
			dewdrop:AddLine(
	  			'text', "Not signed",
	  			'tooltipTitle', "Not Signed",
				'tooltipText', "Not signed",
				'checked', self.db.profile.notifyNotsigned,
				'func', function()
					self:ToggleNotifyNotsigned()
					self:Update()
					end
			)
		elseif (value == "colorOptions") then
			dewdrop:AddLine(
    			'text', "Delimiter Color",
    			'tooltipTitle', "Delimiter Color",
    			'tooltipText', "Color of the brackets, parenthesis etc.",
    			'hasColorSwatch', true,
    			'hasOpacity', true,
    			'r', self.db.profile.delimiterColor.r,
    			'g', self.db.profile.delimiterColor.g,
    			'b', self.db.profile.delimiterColor.b,
    			'colorFunc', function(r, g, b, a)
    				self:SetDelimiterColor(r, g, b, a)
    				end
			)
			dewdrop:AddLine(
    			'text', "Date Color",
    			'tooltipTitle', "Date Color",
    			'tooltipText', "Event date color",
    			'hasColorSwatch', true,
    			'hasOpacity', true,
    			'r', self.db.profile.dateColor.r,
    			'g', self.db.profile.dateColor.g,
    			'b', self.db.profile.dateColor.b,
    			'colorFunc', function(r, g, b, a)
    				self:SetDateColor(r, g, b, a)
    				end
			)
			dewdrop:AddLine(
    			'text', "Time Color",
    			'tooltipTitle', "Time Color",
    			'tooltipText', "Event time color",
    			'hasColorSwatch', true,
    			'hasOpacity', true,
    			'r', self.db.profile.timeColor.r,
    			'g', self.db.profile.timeColor.g,
    			'b', self.db.profile.timeColor.b,
    			'colorFunc', function(r, g, b, a)
    				self:SetTimeColor(r, g, b, a)
    				end
			)
			dewdrop:AddLine(
    			'text', "Title Color",
    			'tooltipTitle', "Title Color",
    			'tooltipText', "Event title color",
    			'hasColorSwatch', true,
    			'hasOpacity', true,
    			'r', self.db.profile.titleColor.r,
    			'g', self.db.profile.titleColor.g,
    			'b', self.db.profile.titleColor.b,
    			'colorFunc', function(r, g, b, a)
    				self:SetTitleColor(r, g, b, a)
    				end
			)
		end
	end
end

local InviteStatus = {
	"INVITED",
	"ACCEPTED",
	"DECLINED",
	"CONFIRMED",
	"OUT",
	"STANDBY",
	"SIGNED",
	"NOT SIGNED",
	"TENTATIVE",
}

local EventTypeName = {
	"Blizzard",
	"Raid",
	"Dungeon" ,
	"PvP",
	"Meeting",
	"Other",
}

function CalendarFu:MapEventType(eventType)
	if eventType == 0 then
		return "BLIZZARD"
	elseif eventType == 1 then
		return "RAID"
	elseif eventType == 2 then
		return "DUNGEON"
	elseif eventType == 3 then
		return "PVP"
	elseif eventType == 4 then
		return "MEETING"
	elseif eventType == 5 then
		return "OTHER"
	end
end

function CalendarFu:MapInviteStatus(inviteStatus)
	if inviteStatus == 1 then
		return "INVITED"
	elseif inviteStatus == 2 then
		return "ACCEPTED"
	elseif inviteStatus == 3 then
		return "DECLINED"
	elseif inviteStatus == 4 then
		return "CONFIRMED"
	elseif inviteStatus == 5 then
		return "OUT"
	elseif inviteStatus == 6 then
		return "STANDBY"
	elseif inviteStatus == 7 then
		return "SIGNED"
	elseif inviteStatus == 8 then
		return "NOT SIGNED"
    elseif inviteStatus == 9 then
		return "TENTATIVE"
	end
end

function CalendarFu:OnInitialize()

end

function CalendarFu:OnEnable()
	self:ScheduleRepeatingEvent("CalendarFu_RepeatingTimer",self.OnUpdate, 5, self)
end

function CalendarFu:OnDisable()
	self:CancelScheduledEvent("CalendarFu_RepeatingTimer")
end

function CalendarFu:OnClick(button)
	button = button or "LeftButton"
	if button == "LeftButton" then
		if Calendar_Toggle then
			Calendar_Toggle()
		end
	end
end

function CalendarFu:OnUpdate()
	self:OnTextUpdate()
end

function CalendarFu:OnTextUpdate()

	local numEvents = self:UpdateEventInfo()

	local countInvited = 0
	local countAccepted = 0
	local countDeclined = 0
	local countConfirmed = 0
	local countOut = 0
	local countStandby = 0
	local countSigned = 0
	local countNotsigned = 0
	local countTentative = 0
	
	if numEvents > 0 then
		for eventIndex = 1, numEvents, 1 do
			eventType = eventInfo[eventIndex]["evtype"]
			calendarType = eventInfo[eventIndex]["caltype"]
			eventStatus = eventInfo[eventIndex]["invstatus"]

			if eventStatus == 1 then
				countInvited = countInvited + 1
			elseif eventStatus == 2 then
				countAccepted = countAccepted + 1
			elseif eventStatus == 3 then
				countDeclined = countDeclined + 1
			elseif eventStatus == 4 then
				countConfirmed = countConfirmed + 1
			elseif eventStatus == 5 then
				countOut = countOut + 1
			elseif eventStatus == 6 then
				countStandby = countStandby + 1
			elseif eventStatus == 7 then
				countSigned = countSigned + 1
			elseif eventStatus == 8 then
				countNotsigned = countNotsigned + 1
            elseif eventStatus == 9 then
				countTentative = countTentative + 1
			end
		
		end
	end
	
	fubarTitleText = " "

	if countInvited > 0 and countNotsigned <= 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[1]..countInvited.."|r "
	elseif countInvited + countNotsigned > 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[1]..countInvited + countNotsigned.."|r "
	end
	
	if countAccepted > 0 and countSigned <= 0 and countConfirmed <= 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[2]..countAccepted.."|r "
	elseif countAccepted + countSigned > 0 and countConfirmed <= 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[2]..countAccepted + countSigned.."|r "
	elseif countAccepted + countConfirmed > 0 and countSigned <= 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[2]..countAccepted + countConfirmed.."|r "
	elseif countAccepted + countSigned + countConfirmed > 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[2]..countAccepted + countSigned + countConfirmed.."|r "
	end
	
    if countTentative > 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[9]..countTentative.."|r "
	end
	
	if countDeclined > 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[3]..countDeclined.."|r "
	end
	
	if countOut > 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[5]..countOut.."|r "
	end
	
	if countStandby > 0 then
		fubarTitleText = fubarTitleText..ColorEventInv[6]..countStandby.."|r "
	end
	
	if countInvited > 0 or countAccepted > 0 or countDeclined > 0 or countConfirmed > 0 or countOut > 0 or countStandby > 0 or countSigned > 0 or countNotsigned > 0 or countTentative > 0 then
		self:SetText(fubarTitleText)
	else
		self:SetText(" |cffffffffCalendar|r|cff00ff00Fu|r")
	end
	
	if countInvited > 0 and countNotsigned <= 0 then
		self:SetIcon(iconFiles.iconnew)
	elseif countInvited + countNotsigned > 0 then
		self:SetIcon(iconFiles.iconnew)
	else
		self:SetIcon(iconFiles.icon)
	end

end

function CalendarFu:OnTooltipUpdate()
	local numEvents = self:UpdateEventInfo()
	local eventDate = ""
	
	local cat = tablet:AddCategory(
	'columns', 3
	)
	for eventIndex = 1, numEvents, 1 do
		eventTime = eventInfo[eventIndex]["time"]
		eventTitle = eventInfo[eventIndex]["title"]
		eventType = eventInfo[eventIndex]["evtype"]
		eventStatus = eventInfo[eventIndex]["invstatus"]
		
		if eventDate ~= eventInfo[eventIndex]["date"] then
			eventDate = eventInfo[eventIndex]["date"]
			cat:AddLine(
				'text', " "
			)
			cat:AddLine(
				'text', self:TextColorFormat(self.db.profile.dateColor)..eventDate.."|r",
				'size', 14
			)
		end
		cat:AddLine(
			'text', self:TextColorFormat(self.db.profile.delimiterColor).." ["
				  ..self:TextColorFormat(self.db.profile.timeColor)..eventTime
				  ..self:TextColorFormat(self.db.profile.delimiterColor).."] "
				  ..self:TextColorFormat(self.db.profile.titleColor)..eventTitle.."|r",
			'text2', "|cffffffff"..EventTypeName[eventType + 1].."|r",
			'text3', ColorEventInv[eventStatus]..InviteStatus[eventStatus].."|r"
		)
	end

	local hint = ""
	local clickColor = "|cffeda55f"
	hint = hint..newLine..format("%s to Show or Hide Blizzard Calendar.", clickColor.."Left click"..endColor)
			   ..newLine..format("%s to open Options menu.", clickColor.."Right click"..endColor)
	
	tablet:SetHint(hint)
	
end

------------------------------------------
-- Update events
------------------------------------------
function CalendarFu:UpdateEventInfo()

	if not IsAddOnLoaded("Blizzard_Calendar") then
		local loaded, reason = LoadAddOn("Blizzard_Calendar")
		if not loaded then
			return 0
		else
			if Calendar_Show then
				Calendar_Show()
			end
			if Calendar_Hide then
				Calendar_Hide()
			end
		end
	end
	
	eventInfo = {}
	
	local monthOffset = 0

	local _, month, day, year = CalendarGetDate();

 	for dayOffset = 0, self.db.profile.fordays, 1 do
 	
 		local _, _, numDays = CalendarGetMonth();
 		
 		if day+dayOffset > numDays then
 			dayOffset = dayOffset-numDays
 			monthOffset = 1
 			_, _, numDays = CalendarGetMonth(monthOffset);
 		end
 		
		local numEvents = CalendarGetNumDayEvents(monthOffset, day+dayOffset)
 		
 		if numEvents > 0 then	
			
			local eventIndex = 1
			
			for eventIndex = 1, numEvents, 1 do

				local title, hour, minute, calendarType, _, eventType, _, _, inviteStatus = CalendarGetDayEvent(monthOffset, day+dayOffset, eventIndex)
				
				if title and hour and minute and eventType and inviteStatus then

					local eventTypeIndex = self:MapEventType(eventType)
					
					if eventTypeIndex == -1 then
						break
					end
					
					local eventTypeSelected = false
					
					if self.db.profile.evBlizzard and eventTypeIndex == "BLIZZARD" then
						eventTypeSelected = true
					elseif self.db.profile.evRaid and eventTypeIndex == "RAID" then
						eventTypeSelected = true
					elseif self.db.profile.evDungeon and eventTypeIndex == "DUNGEON" then
						eventTypeSelected = true
					elseif self.db.profile.evPvP and eventTypeIndex == "PVP" then
						eventTypeSelected = true
					elseif self.db.profile.evMeeting and eventTypeIndex == "MEETING" then
						eventTypeSelected = true
					elseif self.db.profile.evOther and eventTypeIndex == "OTHER" then
						eventTypeSelected = true
					end
					
					local calendarTypeSelected = false
					
					if self.db.profile.calPlayer and calendarType == "PLAYER" then
						calendarTypeSelected = true
					elseif self.db.profile.calGuild and (calendarType == "GUILD_ANNOUNCEMENT" or calendarType == "GUILD_EVENT") then
						calendarTypeSelected = true
					elseif self.db.profile.calArena and calendarType == "ARENA" then
						calendarTypeSelected = true
					elseif self.db.profile.calHoliday and calendarType == "HOLIDAY" then
						calendarTypeSelected = true
					end
					
					local inviteTypeSelected = false
					
					if self.db.profile.notifyInvited and inviteStatus == 1 then
						inviteTypeSelected = true
					elseif self.db.profile.notifyAccepted and inviteStatus == 2 then
						inviteTypeSelected = true
					elseif self.db.profile.notifyDeclined and inviteStatus == 3 then
						inviteTypeSelected = true
					elseif self.db.profile.notifyConfirmed and inviteStatus == 4 then
						inviteTypeSelected = true
					elseif self.db.profile.notifyOut and inviteStatus == 5 then
						inviteTypeSelected = true
					elseif self.db.profile.notifyStandby and inviteStatus == 6 then
						inviteTypeSelected = true
					elseif self.db.profile.notifySigned and inviteStatus == 7 then
						inviteTypeSelected = true
					elseif self.db.profile.notifyNotsigned and inviteStatus == 8 then
						inviteTypeSelected = true
                    elseif self.db.profile.notifyTentative and inviteStatus == 9 then
						inviteTypeSelected = true
					end
					
					if eventTypeSelected == true and calendarTypeSelected == true and inviteTypeSelected == true then
						local nextIndex = #eventInfo + 1
						eventInfo[nextIndex] = {}
						eventInfo[nextIndex]["evtype"] = eventType
						eventInfo[nextIndex]["caltype"] = calendarType
						eventInfo[nextIndex]["invstatus"] = inviteStatus
						eventInfo[nextIndex]["time"] = string.format("%02d:%02d", hour, minute)
						eventInfo[nextIndex]["date"] = string.format("%02d.%02d.%04d", day+dayOffset, month+monthOffset, year)
						eventInfo[nextIndex]["title"] = title
					end

				end
			end
		end
	end
	
	return #eventInfo
	
end

------------------------------------------
-- OnMenu functions
------------------------------------------
function CalendarFu:IsEvBlizzard()
	return self.db.profile.evBlizzard
end

function CalendarFu:ToggleEvBlizzard()
	self.db.profile.evBlizzard = not self.db.profile.evBlizzard
	self:Update()
end

function CalendarFu:IsEvRaid()
	return self.db.profile.evRaid
end

function CalendarFu:ToggleEvRaid()
	self.db.profile.evRaid = not self.db.profile.evRaid
	self:Update()
end

function CalendarFu:IsEvDungeon()
	return self.db.profile.evDungeon
end

function CalendarFu:ToggleEvDungeon()
	self.db.profile.evDungeon = not self.db.profile.evDungeon
	self:Update()
end

function CalendarFu:IsEvPvP()
	return self.db.profile.evPvP
end

function CalendarFu:ToggleEvPvP()
	self.db.profile.evPvP = not self.db.profile.evPvP
	self:Update()
end

function CalendarFu:IsEvMeeting()
	return self.db.profile.evMeeting
end

function CalendarFu:ToggleEvMeeting()
	self.db.profile.evMeeting = not self.db.profile.evMeeting
	self:Update()
end

function CalendarFu:IsEvOther()
	return self.db.profile.evOther
end

function CalendarFu:ToggleEvOther()
	self.db.profile.evOther = not self.db.profile.evOther
	self:Update()
end

function CalendarFu:IsCalPlayer()
	return self.db.profile.calPlayer
end

function CalendarFu:ToggleCalPlayer()
	self.db.profile.calPlayer = not self.db.profile.calPlayer
	self:Update()
end

function CalendarFu:IsCalGuild()
	return self.db.profile.calGuild
end

function CalendarFu:ToggleCalGuild()
	self.db.profile.calGuild = not self.db.profile.calGuild
	self:Update()
end

function CalendarFu:IsCalArena()
	return self.db.profile.calArena
end

function CalendarFu:ToggleCalArena()
	self.db.profile.calArena = not self.db.profile.calArena
	self:Update()
end

function CalendarFu:IsCalHoliday()
	return self.db.profile.calHoliday
end

function CalendarFu:ToggleCalHoliday()
	self.db.profile.calHoliday = not self.db.profile.calHoliday
	self:Update()
end

function CalendarFu:IsNotifyInvited()
	return self.db.profile.notifyInvited
end

function CalendarFu:ToggleNotifyInvited()
	self.db.profile.notifyInvited = not self.db.profile.notifyInvited
	self:Update()
end

function CalendarFu:IsNotifyAccepted()
	return self.db.profile.notifyAccepted
end

function CalendarFu:ToggleNotifyAccepted()
	self.db.profile.notifyAccepted = not self.db.profile.notifyAccepted
	self:Update()
end

function CalendarFu:IsNotifyDeclined()
	return self.db.profile.notifyDeclined
end

function CalendarFu:ToggleNotifyDeclined()
	self.db.profile.notifyDeclined = not self.db.profile.notifyDeclined
	self:Update()
end

function CalendarFu:IsNotifyConfirmed()
	return self.db.profile.notifyConfirmed
end

function CalendarFu:ToggleNotifyConfirmed()
	self.db.profile.notifyConfirmed = not self.db.profile.notifyConfirmed
	self:Update()
end

function CalendarFu:IsNotifyOut()
	return self.db.profile.notifyOut
end

function CalendarFu:ToggleNotifyOut()
	self.db.profile.notifyOut = not self.db.profile.notifyOut
	self:Update()
end

function CalendarFu:IsNotifyStandby()
	return self.db.profile.notifyStandby
end

function CalendarFu:ToggleNotifyStandby()
	self.db.profile.notifyStandby = not self.db.profile.notifyStandby
	self:Update()
end

function CalendarFu:IsNotifySigned()
	return self.db.profile.notifySigned
end

function CalendarFu:ToggleNotifySigned()
	self.db.profile.notifySigned = not self.db.profile.notifySigned
	self:Update()
end

function CalendarFu:IsNotifyNotsigned()
	return self.db.profile.notifyNotsigned
end

function CalendarFu:ToggleNotifyNotsigned()
	self.db.profile.notifyNotsigned = not self.db.profile.notifyNotsigned
	self:Update()
end

function CalendarFu:IsNotifyTentative()
	return self.db.profile.notifyTentative
end

function CalendarFu:ToggleNotifyTentative()
	self.db.profile.notifyTentative = not self.db.profile.notifyTentative
	self:Update()
end

function CalendarFu:SetDelimiterColor(r, g, b, a)
	self.db.profile.delimiterColor.r = r
	self.db.profile.delimiterColor.g = g
	self.db.profile.delimiterColor.b = b
	self.db.profile.delimiterColor.a = a
	self:Update()
end

function CalendarFu:SetDateColor(r, g, b, a)
	self.db.profile.dateColor.r = r
	self.db.profile.dateColor.g = g
	self.db.profile.dateColor.b = b
	self.db.profile.dateColor.a = a
	self:Update()
end

function CalendarFu:SetTimeColor(r, g, b, a)
	self.db.profile.timeColor.r = r
	self.db.profile.timeColor.g = g
	self.db.profile.timeColor.b = b
	self.db.profile.timeColor.a = a
	self:Update()
end

function CalendarFu:SetTitleColor(r, g, b, a)
	self.db.profile.titleColor.r = r
	self.db.profile.titleColor.g = g
	self.db.profile.titleColor.b = b
	self.db.profile.titleColor.a = a
	self:Update()
end

------------------------------------------
-- Color converters
------------------------------------------
local function RGBAToHex(r, g, b, a)
	-- WoW r,g,b,a scale is 0.0-1.0 so multiply first
	r = r * 255
	g = g * 255
	b = b * 255
	a = a * 255
	return string.format("%02x%02x%02x%02x", a, r, g, b)
end

function CalendarFu:TextColorFormat(c)
	if c ~= nil then
		return "|c" .. RGBAToHex(c.r, c.g, c.b, c.a)
	else
		return "|c00000000"
	end
end
